@@ -1,4 +1,3 @@
-from yota.exceptions import DataAccessException, NotCallableException
 from yota.renderers import JinjaRenderer
 from yota.processors import FlaskPostProcessor
 from yota.nodes import LeaderNode, Node
@@ -6,46 +5,49 @@ from yota.validators import Check
 import json
 import copy
 
-class TrackingMeta(type):
-    reserved_attr_names = ('context', 'hidden', 'g_context', 'start_template',
-                           'close_template', 'auto_start_close', '_renderer',
-                           '_processor', 'name')
 
-    def __init__(mcs, name, bases, dict):
+class TrackingMeta(type):
+    """ This metaclass builds our Form classes. It generates the internal
+    _node_list which preserves order of Nodes in your Form as declared. It also
+    generates _validation_list for explicitly declared Check attributes in the
+    Form """
+
+    def __init__(mcs, name, bases, dct):
         """ Process all of the attributes in the `Form` (or subclass)
         declaration and place them accordingly. This builds the internal
-        _node_list and _validation_list. """
+        _node_list and _validation_list and is responsible for preserving
+        initial Node order. """
 
-        t = {}
+        nodes = {}
         mcs._validation_list = []
-        for name, value in dict.items():
-            if isinstance(value, Node):
-                if name in TrackingMeta.reserved_attr_names:
-                    raise AttributeError(
-                        '{0} is a forbidden attribute name for a Node because'
-                        ' it overlaps with a Form attribute. Please rename.')
-                value._attr_name = name
-                t[value._create_counter] = value
-                if hasattr(value, 'validators'):
-                    if not isinstance(value.validators, tuple) and \
-                       not isinstance(value.validators, list):
-                        value.validators = [value.validators, ]
-                    for validator in value.validators:
-                        # shorthand for adding a validation tuple
-                        c = Check(validator, name)
-                        mcs._validation_list.append(c)
-            elif isinstance(value, Check):
-                # if we've found a validation tuple
-                value._attr_name = name
-                mcs._validation_list.append(value)
-
         mcs._node_list = []
-        for i, value in sorted(t.items()):
-            # keeps track of the order of items for actual rendering
-            mcs._node_list.append(value)
+        for name, attribute in dct.items():
+            # These aren't ordered Nodes, ignore them
+            if name is 'start' or name is 'close':
+                try:
+                    attribute._attr_name = name
+                except AttributeError:
+                    raise AttributeError("start/close attribute is special and"
+                        "should specify a Node to begin your form. Got type {0}"
+                        "instead".format(type(name)))
+                continue
+            if isinstance(attribute, Node):
+                attribute._attr_name = name
+                nodes[attribute._create_counter] = attribute
+                delattr(mcs, name)
+            elif isinstance(attribute, Check):
+                # if we've found a validation check
+                attribute._attr_name = name
+                mcs._validation_list.append(attribute)
+                delattr(mcs, name)
 
+        # insert our nodes in sorted order by there initialization order, thus
+        # preserving order
+        for i, attribute in sorted(nodes.items()):
+            mcs._node_list.append(attribute)
 
-class Form(object):
+_Form = TrackingMeta('_Form', (object, ), {})
+class Form(_Form):
     """ This is the base class that all user defined forms should inherit from,
     and as such it is the main way to access functionality in Yota. It
     provides the core functionality involved with setting up and
@@ -88,97 +90,67 @@ class Form(object):
     framework being used to a format that Yota expects. It also allows things
     like filtering stripping characters or encoding all data that enters a
     validator. """
+    _reserved_attr_names = ('context', 'hidden', 'g_context', 'start_template',
+                        'close_template', 'auto_start_close', '_renderer',
+                        '_processor', 'name')
+    name = None
+    context = {}
+    g_context = {}
+    title = None
+    auto_start_close = True
+    start_template = 'form_open'
+    close_template = 'form_close'
+    render_success = False
+    render_error = False
 
-    def __new__(cls, **kwargs):
-        """ We want our created Form to have a copy of the original
-        form list so that dynamic additions to the list do not
-        effect all Form instances """
+    def __init__(self, **kwargs):
+        # A bit of a hack to copy all our class attributes
+        for class_attr in dir(self):
+            if class_attr in kwargs:
+                continue
+            att = getattr(self, class_attr)
+            # We want to copy all the nodes as well as the list, this is a
+            # succinct way to do it
+            if class_attr in ['_node_list', '_validation_list']:
+                setattr(self, class_attr, copy.deepcopy(att))
+            # Private attributes are internal stuff..
+            elif not class_attr.startswith('__'):
+                # don't try to copy functions, it doesn't go well
+                if not callable(att):
+                    setattr(self, class_attr, copy.copy(att))
+                    self.context[class_attr] = att
 
-        c = super(Form, cls).__new__(cls, **kwargs)
-        c._node_list = copy.deepcopy(cls._node_list)
-        for n in c._node_list:
-            setattr(c, n._attr_name, n)
-        c._validation_list = copy.deepcopy(cls._validation_list)
-        for n in c._validation_list:
-            if n._attr_name:
-                setattr(c, n._attr_name, n)
-        return c
+        # Set a default name for our Form
+        if self.name is None:
+            self.name = self.__class__.__name__
 
-    def __init__(self,
-                 name=None,
-                 auto_start_close=None,
-                 start_template=None,
-                 close_template=None,
-                 g_context=None,
-                 context=None,
-                 start=None,
-                 close=None,
-                 **kwargs):
+        # pass some attributes to start/close nodes
+        self.context['name'] = self.name
+        self.context['title'] = self.title
 
-        """ Basically, set the instance attribute to one of the following in
-        order of preference:
-        1. Passed in parameter
-        2. Class attribute
-        3. Set default """
-        def override(value, attr, default):
-            if value:
-                setattr(self, attr, value)
-            elif not hasattr(self, attr):
-                setattr(self, attr, default)
+        # run our safety checks, set identifiers, and set local attributes
+        for node in self._node_list:
+            self._setup_node(node)
 
-        # override semantics
-        override(auto_start_close, 'auto_start_close', True)
-        override(start_template, 'start_template', 'form_open')
-        override(close_template, 'close_template', 'form_close')
-        override(g_context, 'g_context', {})
-        override(context, 'context', {})
-
-        # set a default for our name to the class name
-        self.name = name if name else self.__class__.__name__
-        self.context['name'] = self.name  # pass it to start/close
-
-        # pass some special keywords to our context if they're defined as class
-        # attributes
-        if hasattr(self, 'title'):
-            self.context['title'] = self.title
         # passes everything to our rendering context and updates params.
-        # Overwrites class attributes
         self.context.update(kwargs)
-
-        # since our default id is based off of the parent id
-        # we can pass it in here
-        for n in self._node_list:
-            n.set_identifiers(self.name)
+        self.__dict__.update(kwargs)
 
         # Add our open and close form defaults
-        if not start and not hasattr(self, 'start'):
+        if hasattr(self, 'start'):
+            self._node_list.insert(0, self.start)
+        else:
             if self.auto_start_close:
                 self.insert(0, LeaderNode(template=self.start_template,
                                         _attr_name='start',
                                         **self.context))
+        if hasattr(self, 'close'):
+            self._node_list.append(self.close)
         else:
-            # prefer a parameter over a class attr
-            ins = start if start else self.start
-            if isinstance(ins, Node):
-                self.insert(0, ins)
-            else:
-                raise AttributeError("start attribute is special and should "
-                                     "specify a Node to begin your form")
-
-        # do the same for close
-        if not close and not hasattr(self, 'close'):
             if self.auto_start_close:
                 self.insert(-1, LeaderNode(template=self.close_template,
-                                        _attr_name='close',
-                                        **self.context))
-        else:
-            # prefer a parameter over a class attr
-            ins = close if close else self.close
-            if isinstance(ins, Node):
-                self.insert(-1, ins)
-            else:  # provide a bit of error notif
-                raise AttributeError("close attribute is special and should "
-                                     "specify a Node to begin your form")
+                                           _attr_name='close',
+                                           **self.context))
 
         # Add some useful global variables for templates
         default_globals = {'form_id': self.name}
@@ -190,7 +162,6 @@ class Form(object):
         self._last_valid = None
         self._last_raw_json = None
 
-
     def render(self):
         """ Runs the renderer to parse templates of nodes and generate the form
         HTML.
@@ -200,26 +171,64 @@ class Form(object):
 
         return self._renderer().render(self._node_list, self.g_context)
 
+    def _setup_node(self, node):
+        """ An internal function performs some safety checks, sets attribute,
+        and set_identifiers """
+        try:
+            if type(node._attr_name) is not str:
+                raise AttributeError
+        except AttributeError as e:
+            raise AttributeError('Dynamically inserted nodes must have a _attr_name'
+                                 ' attribute as a string. Please add it. ')
+
+        if hasattr(self, node._attr_name):
+            raise AttributeError( 'Attribute name {0} overlaps with a Form '
+                                 'attribute. Please rename.'
+                .format(node._attr_name))
+
+        node.set_identifiers(self.name)
+        setattr(self, node._attr_name, node)
+
+    def _parse_shorthand_validator(self, node):
+        """ Loops thorugh all the Nodes and checks for shorthand validators.
+        After inserting their checks into the form obj they are removed from
+        the node. This is because a validation may be called multiple times on
+        a single form instance. """
+        if hasattr(node, 'validators') and node.validators:
+            # Convert a single callable to an iterator for convenience
+            if callable(node.validators):
+                node.validators = (node.validators, )
+
+            for validator in node.validators:
+                # If they provided a check add it, otherwise make the check
+                # for them
+                if isinstance(validator, Check):
+                    # Just for extra flexibility, add the attr if they left it out
+                    if not validator.args and not validator.kwargs:
+                        validator.args.append(node._attr_name)
+                    self._validation_list.append(validator)
+                else:
+                    # Assume only a single attr if not specified
+                    new_valid = Check(validator, node._attr_name)
+                    self._validation_list.append(new_valid)
+
+            # remove the attribute so multiple calls doesn't break things
+            delattr(node, 'validators')
+
     def insert_validator(self, new_validators):
         """ Inserts a validator to the validator list.
 
         :param validator: The :class:`Check` to be inserted.
         :type validator: Check """
 
-        # Check to make sure the passed in value is a list/tuple
-        if not isinstance(new_validators, tuple) and\
-           not isinstance(new_validators, list):
-            new_validators = [new_validators,]
-
         for validator in new_validators:
             # check to allow passing in just a check
             if not isinstance(validator, Check):
-                raise TypeError
+                raise TypeError('Can only insert type Check or derived classes')
 
             # append the validator to the list
             self._validation_list.append(validator)
 
-
     def insert(self, position, new_node_list):
         """ Inserts a :class:`Node` object or a list of objects at the
         specified position into the :attr:`Form._node_list` of the form.
@@ -235,22 +244,12 @@ class Form(object):
 
         for i, new_node in enumerate(new_node_list):
 
-            # Check to prevent shooting yourself in the foot
-            if new_node._attr_name in TrackingMeta.reserved_attr_names:
-                raise AttributeError('{0} is a forbidden attribute name for a'
-                    'Node because it overlaps with a Form attribute. Please rename.')
-
-            # Another clarity error message
-            if not new_node._attr_name:
-                raise AttributeError('Dynamically added nodes should have an '
-                                     '_attr_name attribute.')
+            self._setup_node(new_node)
 
             if position == -1:
                 self._node_list.append(new_node)
             else:
                 self._node_list.insert(position + i, new_node)
-            setattr(self, new_node._attr_name, new_node)
-            new_node.set_identifiers(self.name)
 
     def insert_after(self, prev_attr_name, new_node_list):
         """ Runs through the internal node structure attempting to find
@@ -290,12 +289,12 @@ class Form(object):
         try:
             attr = getattr(self, name)
         except AttributeError:
-            raise AttributeError('Form attribute {0} couldn\'t be resolved to'
-                                 ' a Node'.format(name))
-        if isinstance(attr, Node):
-            return attr
+            pass
+        else:
+            if isinstance(attr, Node):
+                return attr
         raise AttributeError('Form attribute {0} couldn\'t be resolved to'
-                                ' a Node'.format(name))
+                             ' a Node'.format(name))
 
     def success_header_generate(self):
         """ Please see the documentation for :meth:`Form.error_header_generate`
@@ -304,21 +303,21 @@ class Form(object):
 
     def error_header_generate(self, errors, block):
         """ This function, along with success_header_generate allow you to give
-        form wide information back to the user for both AJAX validated forms
+        form wide information back to the user for both AJAJ validated forms
         and conventionally validated forms, although the mechanisms are
         slightly different. Both functions are run at the end of a successful
         or failed validation call in order to give more information for
         rendering.
 
-        For passing information to AJAX rendering, simply return a dictionary,
-        or any python object that can be serialized to JSON. This information
-        gets passed back to the JavaScript callbacks of yota_activate in
-        slightly different ways. success_header_generate's information will get
-        passed to the render_success callback, while error_header_generate will
-        get sent as an error to the render_error callback under the context
-        start.
+        For passing information to AJAJ rendering, simply return a dictionary,
+        or any Python object that can be serialized to JSON. This information
+        gets passed back to the JavaScript callbacks of yota_activate, however
+        each in slightly different ways. success_header_generate's information
+        will get passed to the render_success callback, while
+        error_header_generate will get sent as an error to the render_error
+        callback under the context start.
 
-        For passing information into a regular rendering context simply access
+        For passing information into a regular, non AJAJ context simply access
         the attribute manually similar to below.
 
         .. code-block:: python
@@ -366,11 +365,14 @@ class Form(object):
         # Allows user to set a modular processor on incoming data
         data = self._processor().filter_post(data)
 
+
         # reset all error lists and data
         for node in self._node_list:
             node.errors = []
             node.data = ''
             node.resolve_data(data)
+            # Pull out all our shorthand validators
+            self._parse_shorthand_validator(node)
 
         # try to load our visited list of it's piecewise validation
         if '_visited_names' not in data and piecewise:
@@ -390,6 +392,9 @@ class Form(object):
                 # If even a single check can't be run, we need to block
                 block = True
 
+        # Run the one off validation method
+        self.validator()
+
         # a list to hold Nodes that actually have errors
         error_node_list = []
         for node in self._node_list:
@@ -411,18 +416,20 @@ class Form(object):
 
         :param piecewise: If set to True, the validator will silently
             ignore validator for which it has insufficient information. This
-            is designed to be used for the AJAX piecewise validation
+            is designed to be used for the AJAJ piecewise validation
             function, although it does not have to be.
         :type piecewise: boolean
 
         :param raw: If set to True then the second return parameter will be a
-        Python dictionary instead of a JSON string
+            Python dictionary instead of a JSON string
+        :type raw: boolean
 
         :return: A boolean whether or not the form submission is valid and the
-        json string (or raw dictionary) to pass back to the javascript side.
-        The boolean is an anding of submission (whether the submit button was
-        actually pressed) and the block parameter (whether or not any blocking
-        validators passed) """
+            json string (or raw dictionary) to pass back to the javascript side.
+            The boolean is an anding of submission (whether the submit button was
+            actually pressed) and the block parameter (whether or not any blocking
+            validators passed)
+        """
 
         # Allows user to set a modular processor on incoming data
         data = self._processor().filter_post(data)
@@ -455,8 +462,8 @@ class Form(object):
             blob = self.success_header_generate()
             if blob:
                 retval['success_blob'] = blob
-                if hasattr(self, 'start'):
-                    retval['success_ids'] = self.start.json_identifiers()
+            if hasattr(self, 'start'):
+                retval['success_ids'] = self.start.json_identifiers()
 
         retval['errors'] = errors
 
@@ -482,7 +489,8 @@ class Form(object):
         """ Runs all the validators associated with the :class:`Form`.
 
         :return: Whether the validators are blocking submission and a list of
-        nodes that have validation messages """
+            nodes that have validation messages.
+        """
 
         # Allows user to set a modular processor on incoming data
         data = self._processor().filter_post(data)
@@ -505,7 +513,8 @@ class Form(object):
         :type data: dictionary
 
         :return: Whether the validators are blocking submission and a re-render
-        of the form with the validation data passed in """
+            of the form with the validation data passed in.
+        """
 
         # Allows user to set a modular processor on incoming data
         data = self._processor().filter_post(data)
@@ -524,6 +533,13 @@ class Form(object):
             self.error_header_generate(invalid, block)
         return (not block), self.render()
 
+    def validator(self):
+        """ This is provided as a convenience method for Validation logic that
+        is one-off, and only intended for a single form. Simply override this
+        function and access any of your Nodes and their data via the self. This
+        method will be called after all other Validators are run. """
+        pass
+
     def update_success(self, update_dict, raw=False):
         """ This method serves as an easy way to update your success attributes
         that are passed to the start Node rendering context, or passed back in
@@ -539,15 +555,16 @@ class Form(object):
         :type raw: bool
 
         :return: Return value is either the new JSON string (or raw dict if
-        requested) if json_validate was your last validation call, or a
-        re-render of the form with updated error messages if validate_render
-        was your last call.  """
+            requested) if json_validate was your last validation call, or a
+            re-render of the form with updated error messages if validate_render
+            was your last call.
+        """
 
         if self._last_valid == 'render':
             try:
                 self.start.errors[-1].update(update_dict)
-            except KeyError:
-                raise KeyError("Error updating your error dictionary for the "
+            except IndexError:
+                raise IndexError("Error updating your error dictionary for the "
                                "start Node. There were no errors to modify.")
             except AttributeError:
                 raise AttributeError("This method is designed to update an "
