@@ -1,7 +1,7 @@
-import logging
 import re
 from yota.exceptions import NotCallableException
 
+
 class MinLengthValidator(object):
     """ Checks to see if data is at least length long.
 
@@ -20,7 +20,7 @@ class MinLengthValidator(object):
         super(MinLengthValidator, self).__init__()
 
     def __call__(self, target):
-        if len(str(target.data)) < self.min_length:
+        if len(target.data) < self.min_length:
             target.add_error({'message': self.message})
 
 
@@ -42,7 +42,7 @@ class MaxLengthValidator(object):
         super(MaxLengthValidator, self).__init__()
 
     def __call__(self, target):
-        if len(str(target.data)) > self.max_length:
+        if len(target.data) > self.max_length:
             target.add_error({'message': self.message})
 
 
@@ -87,7 +87,7 @@ class IntegerValidator(object):
 
     def __call__(self, target):
         try:
-            target.data = int(target.data)
+            int(target.data)
         except ValueError:
             target.add_error({'message': self.message})
 
@@ -110,14 +110,14 @@ class MinMaxValidator(object):
         if message:
             self.message = message
         else:
-            self.message = "Length must be between {0} and {1} characters.". \
-                            format(min, max)
+            self.message = "Length must be between {0} and {1} characters.".\
+                format(min, max)
         self.min = min
         self.max = max
         super(MinMaxValidator, self).__init__()
 
     def __call__(self, target):
-        if len(str(target.data)) < self.min or len(str(target.data)) > self.max:
+        if len(target.data) < self.min or len(target.data) > self.max:
             target.add_error({'message': self.message})
 
 
@@ -138,7 +138,7 @@ class RegexValidator(object):
         super(RegexValidator, self).__init__()
 
     def __call__(self, target=None):
-        if re.match(self.regex, target.data) == None:
+        if re.match(self.regex, target.data) is None:
             target.add_error({'message': self.message})
 
 class PasswordValidator(RegexValidator):
@@ -192,10 +192,10 @@ class StrongPasswordValidator(object):
         self.message = message
         if not isinstance(regex, list):
             self.regex = [
-                 "(?=.*[A-Z].*[A-Z])", # Matches 2 uppercase letters
-                 "(?=.*[!@#$&*])", # Matches 1 Special character
-                 "(?=.*[0-9].*[0-9])", # Matches 2 numbers
-                 ".{7}" # Has at least 7 characters
+                "(?=.*[A-Z].*[A-Z])",  # Matches 2 uppercase letters
+                "(?=.*[!@#$&*])",  # Matches 1 Special character
+                "(?=.*[0-9].*[0-9])",  # Matches 2 numbers
+                ".{7}"  # Has at least 7 characters
              ]
         else:
             self.regex = regex
@@ -255,6 +255,24 @@ class RequiredValidator(object):
         if len(target.data) == 0:
             target.add_error({'message': self.message})
 
+class MimeTypeValidator(object):
+    """ Checks to make sure a posted file is an allowed mime type
+
+    :param message: (optional) The message to present to the user upon failure.
+    :type message: string
+    :param mimetypes: MIME types to check the post against ala 'image/jpeg'
+    :type mimetypes: list
+    """
+    __slots__ = ["message", "mimetypes"]
+
+    def __init__(self, mimetypes, message=None):
+        self.mimetypes = mimetypes
+        self.message = message if message else "Sorry, that MIME type is not supported"
+        super(MimeTypeValidator, self).__init__()
+
+    def __call__(self, target=None):
+        if not target.data.type in self.mimetypes:
+            target.add_error({'message': self.message})
 
 class EmailValidator(object):
     """ A direct port of the Django Email validator. Checks to see if an
@@ -364,7 +382,7 @@ class Check(object):
             self.args[key] = form.get_by_attr(arg)
 
         # Process kwargs
-        for key, val in self.kwargs.iteritems():
+        for key, val in self.kwargs.items():
             self.kwargs[key] = form.get_by_attr(val)
 
         self.resolved = True
@@ -387,7 +405,7 @@ class Check(object):
                 return False
 
         # Process kwargs
-        for node in self.kwargs.itervalues():
+        for node in self.kwargs.values():
             for name in node.get_list_names():
                 if name in visited:
                     break
@@ -411,3 +429,10 @@ class Check(object):
             raise NotCallableException(
                 "Validators provided must be callable, type '{0}'" +
                 "instead. Caused by {1}".format(type(self.validator), e))
+
+    def __iter__(self):
+        """ A simple way to make functions accept lists or single elements """
+        yield self
+
+    def __repr__(self):
+        return "<Check at {0}, args: {1}, kwargs: {2}>".format(id(self), self.args, self.kwargs)
